package com.freshcodes.red5pro;

import android.content.Context;
import android.hardware.Camera;
import android.util.Log;
import android.view.Surface;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.WindowManager;

import androidx.annotation.NonNull;

import com.red5pro.streaming.R5Connection;
import com.red5pro.streaming.R5Stream;
import com.red5pro.streaming.config.R5Configuration;
import com.red5pro.streaming.source.R5Camera;
import com.red5pro.streaming.source.R5Microphone;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;

public class Red5StreamView implements PlatformView {
    int bitrate;
    int fps;

    private final SurfaceView surfaceView;
    private final SurfaceHolder surfaceHolder;
    private Camera camera;
    int surfaceHeight, surfacewidth;

    private int cameraId = Camera.CameraInfo.CAMERA_FACING_FRONT;

    protected boolean isPublishing = false;
    String streamName;
    int recordType;
    protected R5Camera r5Camera;
    protected R5Stream stream;

    Red5StreamView(@NonNull final Context context, BinaryMessenger messenger, int id, final int height, final int width, int bitrate, int fps) {
        this.bitrate = bitrate;
        this.fps = fps;

        surfaceView = new SurfaceView(context);
        surfaceHolder = surfaceView.getHolder();
        surfaceHolder.addCallback(new SurfaceHolder.Callback() {
            @Override
            public void surfaceCreated(@NonNull SurfaceHolder holder) {
                camera = Camera.open(cameraId);
                camera.setDisplayOrientation(getCameraDisplayOrientation(context, cameraId));
                try {
                    camera.setPreviewDisplay(holder);
                } catch (IOException exception) {
                    camera.release();
                    camera = null;
                }
            }

            @Override
            public void surfaceChanged(@NonNull SurfaceHolder holder, int format, int w, int h) {
                surfacewidth = w;
                surfaceHeight = h;

                Camera.Parameters parameters = camera.getParameters();

                Camera.Size optimalSize = R5Camera.getBestPreviewSize(surfaceHeight, surfacewidth, parameters);
                parameters.setPreviewSize(optimalSize.width, optimalSize.height);

                camera.setParameters(parameters);
                camera.startPreview();
            }

            @Override
            public void surfaceDestroyed(@NonNull SurfaceHolder holder) {

            }
        });

        MethodChannel methodChannel = new MethodChannel(messenger, "Red5StreamView/" + id);
        methodChannel.setMethodCallHandler(new MethodChannel.MethodCallHandler() {
            @Override
            public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
                switch (call.method) {
                    case "switchCamera":
                        switchCamera(context);
                        result.success(true);
                        break;
                    case "isPublishing":
                        result.success(isPublishing());
                        break;
                    case "startStreaming":
                        Map<String, Object> configuration = call.hasArgument("configuration") ? (Map<String, Object>) call.argument("configuration") : null;
                        String streamKey = call.hasArgument("streamKey") ? (String) call.argument("streamKey") : null;
                        Integer recordType = call.hasArgument("recordType") ? (Integer) call.argument("recordType") : null;
                        if (configuration != null && streamKey != null && recordType != null) {
                            String licenseKey = (String) configuration.get("licenseKey");
                            int protocol = (int) configuration.get("protocol");
                            String host = (String) configuration.get("host");
                            int port = (int) configuration.get("port");
                            String contextName = (String) configuration.get("contextName");
                            float bufferTime = ((Double) configuration.get("bufferTime")).floatValue();
                            R5Configuration r5Configuration = new R5Configuration(AppUtility.intToR5StreamProtocol(protocol), host, port, contextName, bufferTime);
                            r5Configuration.setLicenseKey(licenseKey);

                            if (!isPublishing) {
                                startStreaming(r5Configuration, streamKey, recordType);
                                result.success(true);
                            } else {
                                result.error("ALREADY_PUBLISHING", "Already stream published", "");
                            }
                        } else {
                            result.error("INVALID_PARAMETER", "StreamKey and Type should not be null", "");
                        }
                        break;
                    case "stopStreaming":
                        if (isPublishing) {
                            stopStreaming();
                            result.success(true);
                        } else {
                            result.error("NOT_LIVE_STREAM", "Stream is not live", "");
                        }
                        break;
                    default:
                        result.notImplemented();
                        break;
                }
            }
        });
    }

    @Override
    public View getView() {
        return surfaceView;
    }

    @Override
    public void dispose() {
        if (camera != null) {
            camera.stopPreview();
            camera.release();
        }
    }

    public boolean isPublishing() {
        return isPublishing;
    }

    public void switchCamera(Context context) {
        camera.stopPreview();
        camera.release();
        if (cameraId == Camera.CameraInfo.CAMERA_FACING_BACK) {
            cameraId = Camera.CameraInfo.CAMERA_FACING_FRONT;
        } else {
            cameraId = Camera.CameraInfo.CAMERA_FACING_BACK;
        }
        try {
            camera = Camera.open(cameraId);
            camera.setDisplayOrientation(getCameraDisplayOrientation(context, cameraId));
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            Camera.Parameters param;
            param = camera.getParameters();
            Camera.Size optimalSize = R5Camera.getBestPreviewSize(surfaceHeight, surfacewidth, param);
            param.setPreviewSize(optimalSize.width, optimalSize.height);
            camera.setParameters(param);
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            camera.setPreviewDisplay(surfaceView.getHolder());
        } catch (IOException e) {
            e.printStackTrace();
        }
        if (isPublishing) {
            r5Camera.setCamera(camera);
            if (cameraId == Camera.CameraInfo.CAMERA_FACING_BACK) {
                r5Camera.setOrientation(90);
            } else {
                r5Camera.setOrientation(270);
            }
        }
        camera.startPreview();
    }

    public int getCameraDisplayOrientation(Context context, int cameraId) {
        android.hardware.Camera.CameraInfo info =
                new android.hardware.Camera.CameraInfo();
        android.hardware.Camera.getCameraInfo(cameraId, info);
        WindowManager windowManager = (WindowManager) context
                .getSystemService(Context.WINDOW_SERVICE);
        int rotation = windowManager.getDefaultDisplay()
                .getRotation();
        int degrees = 0;
        switch (rotation) {
            case Surface.ROTATION_0:
                degrees = 0;
                break;
            case Surface.ROTATION_90:
                degrees = 90;
                break;
            case Surface.ROTATION_180:
                degrees = 180;
                break;
            case Surface.ROTATION_270:
                degrees = 270;
                break;
        }

        int result;
        if (info.facing == Camera.CameraInfo.CAMERA_FACING_FRONT) {
            result = (info.orientation + degrees) % 360;
            result = (360 - result) % 360;
        } else {
            result = (info.orientation - degrees + 360) % 360;
        }
        return result;
    }

    public void startStreaming(R5Configuration configuration, String streamName, int recordType) {
        this.streamName = streamName;
        this.recordType = recordType;

        camera.stopPreview();

        stream = new R5Stream(new R5Connection(configuration));
        stream.setView(surfaceView);

        Camera.Size optimalSize = R5Camera.getBestPreviewSize(surfaceHeight, surfacewidth, camera.getParameters());
        r5Camera = new R5Camera(camera, optimalSize.width, optimalSize.height);
        if (cameraId == Camera.CameraInfo.CAMERA_FACING_BACK) {
            r5Camera.setOrientation(90);
        } else {
            r5Camera.setOrientation(270);
        }
        r5Camera.setBitrate(bitrate);
        r5Camera.setFramerate(fps);

        R5Microphone r5Microphone = new R5Microphone();

        stream.attachCamera(r5Camera);
        stream.attachMic(r5Microphone);

        stream.publish(streamName, intToRecordType(recordType));
        camera.startPreview();
        isPublishing = true;
    }

    public void stopStreaming() {
        if (stream != null) {
            stream.stop();
            camera.startPreview();
        }
        isPublishing = false;
    }

    private R5Stream.RecordType intToRecordType(int type) {
        switch (type) {
            case 0:
                return R5Stream.RecordType.Live;
            case 1:
                return R5Stream.RecordType.Record;
            case 2:
                return R5Stream.RecordType.Append;
            default:
                return R5Stream.RecordType.Live;
        }
    }

    /*private Camera.Size getOptimalPreviewSize(List<Camera.Size> sizes, int w, int h) {
        final double ASPECT_TOLERANCE = 0.1;
        double targetRatio = (double) h / w;

        if (sizes == null) return null;

        Camera.Size optimalSize = null;
        double minDiff = Double.MAX_VALUE;

        int targetHeight = h;

        for (Camera.Size size : sizes) {
            double ratio = (double) size.width / size.height;
            if (Math.abs(ratio - targetRatio) > ASPECT_TOLERANCE) continue;
            if (Math.abs(size.height - targetHeight) < minDiff) {
                optimalSize = size;
                minDiff = Math.abs(size.height - targetHeight);
            }
        }

        if (optimalSize == null) {
            minDiff = Double.MAX_VALUE;
            for (Camera.Size size : sizes) {
                if (Math.abs(size.height - targetHeight) < minDiff) {
                    optimalSize = size;
                    minDiff = Math.abs(size.height - targetHeight);
                }
            }
        }
        return optimalSize;
    }*/
}