package com.freshcodes.red5pro;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.View;

import androidx.annotation.NonNull;

import com.red5pro.streaming.R5Connection;
import com.red5pro.streaming.R5Stream;
import com.red5pro.streaming.config.R5Configuration;
import com.red5pro.streaming.event.R5ConnectionEvent;
import com.red5pro.streaming.event.R5ConnectionListener;
import com.red5pro.streaming.view.R5VideoView;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;

public class Red5StreamPlayerView implements PlatformView {
    String streamName;

    R5VideoView videoView;
    protected R5Stream stream;

    Red5StreamPlayerView(@NonNull final Context context, BinaryMessenger messenger, int id, @NonNull R5Configuration configuration, @NonNull String streamName) {
        this.streamName = streamName;
        videoView = new R5VideoView(context);
        stream = new R5Stream(new R5Connection(configuration));
        videoView.attachStream(stream);

        MethodChannel methodChannel = new MethodChannel(messenger, "Red5StreamPlayerView/" + id);
        methodChannel.setMethodCallHandler(new MethodChannel.MethodCallHandler() {
            @Override
            public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
                switch (call.method) {
                    case "play":
                        try {
                            play(result);
                        } catch (Exception e) {
                            e.printStackTrace();
                            result.error("PLAYER_ERROR", e.getMessage(), e.getLocalizedMessage());
                        }
                        break;
                    case "stop":
                        try {
                            stop();
                            result.success(true);
                        } catch (Exception e) {
                            e.printStackTrace();
                            result.error("PLAYER_ERROR", e.getMessage(), e.getLocalizedMessage());
                        }
                        break;
                    case "setVolume":
                        try {
                            if (call.hasArgument("volume") && call.argument("volume") instanceof Double && (double) call.argument("volume") >= 0.0 && (double) call.argument("volume") <= 1.0) {
                                setVolume((double) call.argument("volume"));
                                result.success(true);
                            }
                            result.error("SET_VOLUME_ERROR", "Invalid Value", "Volume should be from 0.0 to 1.0.");
                        } catch (Exception e) {
                            e.printStackTrace();
                            result.error("SET_VOLUME_ERROR", e.getMessage(), e.getLocalizedMessage());
                        }
                        break;
                    default:
                        result.notImplemented();
                        break;
                }
            }
        });
    }

    @Override
    public View getView() {
        return videoView;
    }

    @Override
    public void dispose() {
        stop();
    }

    public void play(final MethodChannel.Result result) {
        if (stream != null) {
            stream.play(streamName);
            new Handler(Looper.getMainLooper()).post(new Runnable() {
                @Override
                public void run() {
                    stream.setListener(new R5ConnectionListener() {
                        @Override
                        public void onConnectionEvent(R5ConnectionEvent r5ConnectionEvent) {
                            Log.e("R5ConnectionListener", r5ConnectionEvent.name());
                            if (r5ConnectionEvent == R5ConnectionEvent.VIDEO_RENDER_START) {
                                result.success(true);
                            }
                        }
                    });
                }
            });
        }
    }

    public void stop() {
        if (stream != null) {
            stream.stop();
        }
    }

    public void setVolume(double volume) {
        stream.audioController.setPlaybackGain((float) volume);
    }
}
