import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:red5pro/red5_configuration.dart';

typedef void R5StreamPlayerViewCallback(
    R5StreamPlayerViewController controller);

class Red5StreamPlayerView extends StatelessWidget {
  static const StandardMessageCodec _decoder = StandardMessageCodec();

  final Red5Configuration red5configuration;
  final String streamName;

  final R5StreamPlayerViewCallback? onR5StreamPlayerViewCreated;

  const Red5StreamPlayerView(
      {required this.red5configuration,
      required this.streamName,
      this.onR5StreamPlayerViewCreated,
      Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final String viewType = 'Red5StreamPlayerView';
    final Map<String, dynamic> creationParams = <String, dynamic>{};
    creationParams['configuration'] = red5configuration.toJson();
    creationParams['streamName'] = streamName;
    switch (defaultTargetPlatform) {
      case TargetPlatform.android:
        return AndroidView(
            viewType: viewType,
            onPlatformViewCreated: _onPlatformViewCreated,
            creationParams: creationParams,
            creationParamsCodec: _decoder);
      case TargetPlatform.iOS:
        return UiKitView(
          viewType: viewType,
          onPlatformViewCreated: _onPlatformViewCreated,
          creationParams: creationParams,
          creationParamsCodec: _decoder,
        );
      default:
        throw UnsupportedError("Unsupported platform view");
    }
  }

  void _onPlatformViewCreated(int id) {
    if (onR5StreamPlayerViewCreated == null) {
      return;
    }
    onR5StreamPlayerViewCreated!(R5StreamPlayerViewController(id));
  }
}

class R5StreamPlayerViewController {
  late MethodChannel _channel;

  R5StreamPlayerViewController(int id) {
    _channel = MethodChannel('Red5StreamPlayerView/$id');
    _channel.setMethodCallHandler(_handleMethod);
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    print(call.method);
  }

  Future<bool> play() async {
    try {
      final bool result = await _channel.invokeMethod('play');
      print("play: $result");
      return result;
    } on PlatformException catch (e) {
      print("Error play: $e.message");
    }
    return false;
  }

  Future<bool> stop() async {
    try {
      final bool result = await _channel.invokeMethod('stop');
      print("stop: $result");
      return result;
    } on PlatformException catch (e) {
      print("Error stop: $e.message");
    }
    return false;
  }

  Future<bool> setVolume(double volume) async {
    try {
      if (Platform.isAndroid) {
        final bool result =
            await _channel.invokeMethod('setVolume', {'volume': volume});
        print("setVolume: $result");
        return result;
      }
      return true;
    } on PlatformException catch (e) {
      print("Error setVolume: $e.message");
    }
    return false;
  }

  MethodChannel get channel => _channel;
}
